@@ -9,6 +9,7 @@
 #include "xs_regex.h"
 #include "xs_time.h"
 #include "xs_set.h"
+#include "xs_match.h"
 
 #include "snac.h"
 
@@ -192,9 +193,7 @@ int timeline_request(snac *snac, char **id, xs_str **wrk, int level)
                         type = "(null)";
                 }
 
-                if (strcmp(type, "Note") == 0 ||
-                    strcmp(type, "Page") == 0 ||
-                    strcmp(type, "Article") == 0) {
+                if (xs_match(type, "Note|Page|Article")) {
                     const char *actor = xs_dict_get(object, "attributedTo");
 
                     /* request (and drop) the actor for this entry */
@@ -441,7 +440,7 @@ int is_msg_for_me(snac *snac, const xs_dict *c_msg)
 {
     const char *type = xs_dict_get(c_msg, "type");
 
-    if (strcmp(type, "Like") == 0 || strcmp(type, "Announce") == 0) {
+    if (xs_match(type, "Like|Announce")) {
         const char *object = xs_dict_get(c_msg, "object");
 
         if (xs_type(object) == XSTYPE_DICT)
@@ -761,6 +760,12 @@ xs_dict *msg_base(snac *snac, const char *type, const char *id,
     xs *did       = NULL;
     xs *published = NULL;
     xs *ntid      = tid(0);
+    const char *obj_id;
+
+    if (xs_type(object) == XSTYPE_DICT)
+        obj_id = xs_dict_get(object, "id");
+    else
+        obj_id = object;
 
     /* generated values */
     if (date && strcmp(date, "@now") == 0) {
@@ -776,8 +781,8 @@ xs_dict *msg_base(snac *snac, const char *type, const char *id,
         }
         else
         if (strcmp(id, "@object") == 0) {
-            if (object != NULL) {
-                did = xs_fmt("%s/%s_%s", xs_dict_get(object, "id"), type, ntid);
+            if (obj_id != NULL) {
+                did = xs_fmt("%s/%s_%s", obj_id, type, ntid);
                 id = did;
             }
             else
@@ -788,7 +793,7 @@ xs_dict *msg_base(snac *snac, const char *type, const char *id,
             /* like @object, but always generate the same id */
             if (object != NULL) {
                 date = xs_dict_get(object, "published");
-                did = xs_fmt("%s/%s", xs_dict_get(object, "id"), type);
+                did = xs_fmt("%s/%s", obj_id, type);
                 id = did;
             }
             else
@@ -967,6 +972,36 @@ xs_dict *msg_actor(snac *snac)
         msg = xs_dict_set(msg, "image", d);
     }
 
+    /* add the metadata as attachments of PropertyValue */
+    xs_dict *metadata = xs_dict_get(snac->config, "metadata");
+    if (xs_type(metadata) == XSTYPE_DICT) {
+        xs *attach = xs_list_new();
+        xs_str *k;
+        xs_str *v;
+
+        while (xs_dict_iter(&metadata, &k, &v)) {
+            xs *d = xs_dict_new();
+
+            xs *k2 = encode_html(k);
+            xs *v2 = NULL;
+
+            if (xs_startswith(v, "https:")) {
+                xs *t = encode_html(v);
+                v2 = xs_fmt("<a href=\"%s\">%s</a>", t, t);
+            }
+            else
+                v2 = encode_html(v);
+
+            d = xs_dict_append(d, "type",  "PropertyValue");
+            d = xs_dict_append(d, "name",  k2);
+            d = xs_dict_append(d, "value", v2);
+
+            attach = xs_list_append(attach, d);
+        }
+
+        msg = xs_dict_set(msg, "attachment", attach);
+    }
+
     return msg;
 }
 
@@ -996,8 +1031,10 @@ xs_dict *msg_undo(snac *snac, char *object)
 /* creates an 'Undo' message */
 {
     xs_dict *msg = msg_base(snac, "Undo", "@object", snac->actor, "@now", object);
+    const char *to;
 
-    msg = xs_dict_append(msg, "to", xs_dict_get(object, "object"));
+    if (xs_type(object) == XSTYPE_DICT && (to = xs_dict_get(object, "object")))
+        msg = xs_dict_append(msg, "to", to);
 
     return msg;
 }
@@ -1540,18 +1577,23 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
             char *in_reply_to = xs_dict_get(object, "inReplyTo");
             xs *wrk           = NULL;
 
-            timeline_request(snac, &in_reply_to, &wrk, 0);
-
-            if (timeline_add(snac, id, object)) {
-                snac_log(snac, xs_fmt("new 'Note' %s %s", actor, id));
-                do_notify = 1;
+            if (!xs_is_null(in_reply_to) && is_hidden(snac, in_reply_to)) {
+                snac_debug(snac, 0, xs_fmt("dropped reply %s to hidden post %s", id, in_reply_to));
             }
+            else {
+                timeline_request(snac, &in_reply_to, &wrk, 0);
 
-            /* if it has a "name" field, it may be a vote for a question */
-            const char *name = xs_dict_get(object, "name");
+                if (timeline_add(snac, id, object)) {
+                    snac_log(snac, xs_fmt("new 'Note' %s %s", actor, id));
+                    do_notify = 1;
+                }
 
-            if (!xs_is_null(name) && *name && !xs_is_null(in_reply_to) && *in_reply_to)
-                update_question(snac, in_reply_to);
+                /* if it has a "name" field, it may be a vote for a question */
+                const char *name = xs_dict_get(object, "name");
+
+                if (!xs_is_null(name) && *name && !xs_is_null(in_reply_to) && *in_reply_to)
+                    update_question(snac, in_reply_to);
+            }
         }
         else
         if (strcmp(utype, "Question") == 0) { /**  **/
@@ -1640,20 +1682,24 @@ int process_input_message(snac *snac, xs_dict *msg, xs_dict *req)
     }
     else
     if (strcmp(type, "Update") == 0) { /** **/
-        if (strcmp(utype, "Person") == 0 || strcmp(utype, "Service") == 0) {
+        if (xs_match(utype, "Person|Service")) { /** **/
             actor_add(actor, xs_dict_get(msg, "object"));
             timeline_touch(snac);
 
             snac_log(snac, xs_fmt("updated actor %s", actor));
         }
         else
-        if (strcmp(utype, "Note") == 0) { /** **/
+        if (xs_match(utype, "Note|Page|Article")) { /** **/
             const char *id = xs_dict_get(object, "id");
 
-            object_add_ow(id, object);
-            timeline_touch(snac);
+            if (object_here(id)) {
+                object_add_ow(id, object);
+                timeline_touch(snac);
 
-            snac_log(snac, xs_fmt("updated post %s", id));
+                snac_log(snac, xs_fmt("updated post %s", id));
+            }
+            else
+                snac_log(snac, xs_fmt("dropped update for unknown post %s", id));
         }
         else
         if (strcmp(utype, "Question") == 0) { /** **/
@@ -1761,7 +1807,7 @@ void process_user_queue_item(snac *snac, xs_dict *q_item)
             if (inbox != NULL) {
                 /* add to the set and, if it's not there, send message */
                 if (xs_set_add(&inboxes, inbox) == 1)
-                    enqueue_output(snac, msg, inbox, 0);
+                    enqueue_output(snac, msg, inbox, 0, 0);
             }
             else
                 snac_log(snac, xs_fmt("cannot find inbox for %s", actor));
@@ -1775,7 +1821,7 @@ void process_user_queue_item(snac *snac, xs_dict *q_item)
             p = shibx;
             while (xs_list_iter(&p, &inbox)) {
                 if (xs_set_add(&inboxes, inbox) == 1)
-                    enqueue_output(snac, msg, inbox, 0);
+                    enqueue_output(snac, msg, inbox, 0, 0);
             }
         }
 
@@ -1859,6 +1905,7 @@ void process_queue_item(xs_dict *q_item)
         xs_str *seckey = xs_dict_get(q_item, "seckey");
         xs_dict *msg   = xs_dict_get(q_item, "message");
         int retries    = xs_number_get(xs_dict_get(q_item, "retries"));
+        int p_status   = xs_number_get(xs_dict_get(q_item, "p_status"));
         xs *payload    = NULL;
         int p_size     = 0;
 
@@ -1867,8 +1914,14 @@ void process_queue_item(xs_dict *q_item)
             return;
         }
 
-        /* deliver */
-        status = send_to_inbox_raw(keyid, seckey, inbox, msg, &payload, &p_size, retries == 0 ? 3 : 8);
+        if (is_instance_blocked(inbox)) {
+            srv_debug(0, xs_fmt("discarded output message to blocked instance %s", inbox));
+            return;
+        }
+
+        /* deliver (if previous error status was a timeout, try now longer) */
+        status = send_to_inbox_raw(keyid, seckey, inbox, msg,
+                    &payload, &p_size, p_status == 599 ? 20 : 3);
 
         if (payload) {
             if (p_size > 64) {
@@ -1892,8 +1945,13 @@ void process_queue_item(xs_dict *q_item)
         if (!valid_status(status)) {
             retries++;
 
+            /* if it's not the first time it fails with a timeout,
+               penalize the server by skipping one retry */
+            if (p_status == status && status == 499)
+                retries++;
+
             /* error sending; requeue? */
-            if (status == 404 || status == 410 || status < 0)
+            if (status == 400 || status == 404 || status == 405 || status == 410 || status < 0)
                 /* explicit error: discard */
                 srv_log(xs_fmt("output message: fatal error %s %d", inbox, status));
             else
@@ -1901,7 +1959,7 @@ void process_queue_item(xs_dict *q_item)
                 srv_log(xs_fmt("output message: giving up %s %d", inbox, status));
             else {
                 /* requeue */
-                enqueue_output_raw(keyid, seckey, msg, inbox, retries);
+                enqueue_output_raw(keyid, seckey, msg, inbox, retries, status);
                 srv_log(xs_fmt("output message: requeue %s #%d", inbox, retries));
             }
         }
