@@ -23,7 +23,7 @@ double disk_layout = 2.7;
 /* storage serializer */
 pthread_mutex_t data_mutex = {0};
 
-int snac_upgrade(d_char **error);
+int snac_upgrade(xs_str **error);
 
 
 int srv_open(char *basedir, int auto_upgrade)
@@ -154,12 +154,34 @@ int user_open(snac *snac, const char *uid)
     memset(snac, '\0', sizeof(struct _snac));
 
     if (validate_uid(uid)) {
-        xs *cfg_file;
+        xs *cfg_file = NULL;
         FILE *f;
 
-        snac->uid = xs_str_new(uid);
+        xs *t = xs_fmt("%s/user/%s", srv_basedir, uid);
 
-        snac->basedir = xs_fmt("%s/user/%s", srv_basedir, uid);
+        if (mtime(t) == 0.0) {
+            /* user folder does not exist; try with a different case */
+            xs *lcuid = xs_tolower_i(xs_dup(uid));
+            xs *ulist = user_list();
+            xs_list *p = ulist;
+            xs_str *v;
+
+            while (xs_list_iter(&p, &v)) {
+                xs *v2 = xs_tolower_i(xs_dup(v));
+
+                if (strcmp(lcuid, v2) == 0) {
+                    snac->uid = xs_dup(v);
+                    break;
+                }
+            }
+        }
+        else
+            snac->uid = xs_str_new(uid);
+
+        if (snac->uid == NULL)
+            return ret;
+
+        snac->basedir = xs_fmt("%s/user/%s", srv_basedir, snac->uid);
 
         cfg_file = xs_fmt("%s/user.json", snac->basedir);
 
@@ -176,7 +198,7 @@ int user_open(snac *snac, const char *uid)
                     fclose(f);
 
                     if (snac->key != NULL) {
-                        snac->actor = xs_fmt("%s/%s", srv_baseurl, uid);
+                        snac->actor = xs_fmt("%s/%s", srv_baseurl, snac->uid);
                         snac->md5   = xs_md5_hex(snac->actor, strlen(snac->actor));
 
                         /* everything is ok right now */
@@ -1121,15 +1143,6 @@ xs_list *timeline_top_level(snac *snac, xs_list *list)
 xs_list *timeline_simple_list(snac *snac, const char *idx_name, int skip, int show)
 /* returns a timeline (with all entries) */
 {
-    int c_max;
-
-    /* maximum number of items in the timeline */
-    c_max = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
-
-    /* never more timeline entries than the configured maximum */
-    if (show > c_max)
-        show = c_max;
-
     xs *idx = xs_fmt("%s/%s.idx", snac->basedir, idx_name);
 
     return index_list_desc(idx, skip, show);
@@ -1139,6 +1152,15 @@ xs_list *timeline_simple_list(snac *snac, const char *idx_name, int skip, int sh
 xs_list *timeline_list(snac *snac, const char *idx_name, int skip, int show)
 /* returns a timeline (only top level entries) */
 {
+    int c_max;
+
+    /* maximum number of items in the timeline */
+    c_max = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
+
+    /* never more timeline entries than the configured maximum */
+    if (show > c_max)
+        show = c_max;
+
     xs *list = timeline_simple_list(snac, idx_name, skip, show);
 
     return timeline_top_level(snac, list);
@@ -1343,6 +1365,13 @@ int is_pinned(snac *user, const char *id)
 }
 
 
+int is_pinned_by_md5(snac *user, const char *md5)
+{
+    xs *fn = xs_fmt("%s/pinned/%s.json", user->basedir, md5);
+    return !!(mtime(fn) != 0.0);
+}
+
+
 int pin(snac *user, const char *id)
 /* pins a message */
 {
@@ -1759,8 +1788,11 @@ void inbox_add_by_actor(const xs_dict *actor)
     char *v;
 
     if (!xs_is_null(v = xs_dict_get(actor, "endpoints")) &&
-        !xs_is_null(v = xs_dict_get(v, "sharedInbox")))
-        inbox_add(v);
+        !xs_is_null(v = xs_dict_get(v, "sharedInbox"))) {
+        /* only collect this inbox if its instance is not blocked */
+        if (!is_instance_blocked(v))
+            inbox_add(v);
+    }
 }
 
 
@@ -2031,13 +2063,16 @@ void enqueue_input(snac *snac, const xs_dict *msg, const xs_dict *req, int retri
 
 
 void enqueue_output_raw(const char *keyid, const char *seckey,
-                        xs_dict *msg, xs_str *inbox, int retries)
+                        xs_dict *msg, xs_str *inbox, int retries, int p_status)
 /* enqueues an output message to an inbox */
 {
     xs *qmsg   = _new_qmsg("output", msg, retries);
     char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
+    xs *ns = xs_number_new(p_status);
+    qmsg = xs_dict_append(qmsg, "p_status", ns);
+
     qmsg = xs_dict_append(qmsg, "inbox",  inbox);
     qmsg = xs_dict_append(qmsg, "keyid",  keyid);
     qmsg = xs_dict_append(qmsg, "seckey", seckey);
@@ -2052,7 +2087,7 @@ void enqueue_output_raw(const char *keyid, const char *seckey,
 }
 
 
-void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries)
+void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries, int p_status)
 /* enqueues an output message to an inbox */
 {
     if (xs_startswith(inbox, snac->actor)) {
@@ -2062,7 +2097,7 @@ void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries)
 
     char *seckey = xs_dict_get(snac->key, "secret");
 
-    enqueue_output_raw(snac->actor, seckey, msg, inbox, retries);
+    enqueue_output_raw(snac->actor, seckey, msg, inbox, retries, p_status);
 }
 
 
@@ -2072,7 +2107,7 @@ void enqueue_output_by_actor(snac *snac, xs_dict *msg, const xs_str *actor, int
     xs *inbox = get_actor_inbox(snac, actor);
 
     if (!xs_is_null(inbox))
-        enqueue_output(snac, msg, inbox, retries);
+        enqueue_output(snac, msg, inbox, retries, 0);
     else
         snac_log(snac, xs_fmt("enqueue_output_by_actor cannot get inbox %s", actor));
 }
