@@ -22,7 +22,7 @@ const char *smileys[] = {
     "%-)",        "&#129322;",
     ":_(",        "&#128546;",
     ":-|",        "&#128528;",
-    "<3",         "&#128147;",
+    "<3",         "&#10084;&#65039;",
     ":facepalm:", "&#129318;",
     ":shrug:",    "&#129335;",
     ":shrug2:",   "&#175;\\_(&#12484;)_/&#175;",
@@ -203,7 +203,7 @@ xs_str *not_really_markdown(const char *content, xs_list **attach)
 
 const char *valid_tags[] = {
     "a", "p", "br", "br/", "blockquote", "ul", "ol", "li", "cite", "small",
-    "span", "i", "b", "u", "pre", "code", "em", "strong", "hr", "img", "del", NULL
+    "span", "i", "b", "u", "s", "pre", "code", "em", "strong", "hr", "img", "del", NULL
 };
 
 xs_str *sanitize(const char *content)
@@ -235,7 +235,7 @@ xs_str *sanitize(const char *content)
 
             if (valid_tags[i]) {
                 /* accepted tag: rebuild it with only the accepted elements */
-                xs *el = xs_regex_match(v, "(src|href|rel|class|target)=\"[^\"]*\"");
+                xs *el = xs_regex_select(v, "(src|href|rel|class|target)=\"[^\"]*\"");
                 xs *s3 = xs_join(el, " ");
 
                 s2 = xs_fmt("<%s%s%s%s>",
@@ -260,7 +260,7 @@ xs_str *sanitize(const char *content)
 }
 
 
-xs_str *encode_html(const char *str)
+xs_str *encode_html_strict(const char *str)
 /* escapes html characters */
 {
     xs_str *encoded = xs_replace(str, "&", "&amp;");
@@ -269,6 +269,15 @@ xs_str *encode_html(const char *str)
     encoded = xs_replace_i(encoded, "\"", "&#34;");
     encoded = xs_replace_i(encoded, "'", "&#39;");
 
+    return encoded;
+}
+
+
+xs_str *encode_html(const char *str)
+/* escapes html characters */
+{
+    xs_str *encoded = encode_html_strict(str);
+
     /* Restore only <br>. Probably safe. Let's hope nothing goes wrong with this. */
     encoded = xs_replace_i(encoded, "&lt;br&gt;", "<br>");
 
