@@ -10,6 +10,7 @@
 #include "xs_curl.h"
 #include "xs_openssl.h"
 #include "xs_socket.h"
+#include "xs_url.h"
 #include "xs_httpd.h"
 #include "xs_mime.h"
 #include "xs_regex.h"
@@ -17,6 +18,7 @@
 #include "xs_time.h"
 #include "xs_glob.h"
 #include "xs_random.h"
+#include "xs_match.h"
 
 #include "snac.h"
 
@@ -77,6 +79,9 @@ double ftime(void)
 int validate_uid(const char *uid)
 /* returns if uid is a valid identifier */
 {
+    if (!uid || *uid == '\0')
+        return 0;
+
     while (*uid) {
         if (!(isalnum(*uid) || *uid == '_'))
             return 0;
@@ -88,7 +93,7 @@ int validate_uid(const char *uid)
 }
 
 
-void srv_debug(int level, xs_str *str)
+void srv_log(xs_str *str)
 /* logs a debug message */
 {
     if (xs_str_in(str, srv_basedir) != -1) {
@@ -96,25 +101,23 @@ void srv_debug(int level, xs_str *str)
         str = xs_replace_i(str, srv_basedir, "~");
     }
 
-    if (dbglevel >= level) {
-        xs *tm = xs_str_localtime(0, "%H:%M:%S");
-        fprintf(stderr, "%s %s\n", tm, str);
+    xs *tm = xs_str_localtime(0, "%H:%M:%S");
+    fprintf(stderr, "%s %s\n", tm, str);
 
-        /* if the ~/log/ folder exists, also write to a file there */
-        xs *dt = xs_str_localtime(0, "%Y-%m-%d");
-        xs *lf = xs_fmt("%s/log/%s.log", srv_basedir, dt);
-        FILE *f;
-        if ((f = fopen(lf, "a")) != NULL) {
-            fprintf(f, "%s %s\n", tm, str);
-            fclose(f);
-        }
+    /* if the ~/log/ folder exists, also write to a file there */
+    xs *dt = xs_str_localtime(0, "%Y-%m-%d");
+    xs *lf = xs_fmt("%s/log/%s.log", srv_basedir, dt);
+    FILE *f;
+    if ((f = fopen(lf, "a")) != NULL) {
+        fprintf(f, "%s %s\n", tm, str);
+        fclose(f);
     }
 
     xs_free(str);
 }
 
 
-void snac_debug(snac *snac, int level, xs_str *str)
+void snac_log(snac *snac, xs_str *str)
 /* prints a user debugging information */
 {
     xs *o_str = str;
@@ -125,7 +128,7 @@ void snac_debug(snac *snac, int level, xs_str *str)
         msg = xs_replace_i(msg, snac->basedir, "~");
     }
 
-    srv_debug(level, msg);
+    srv_log(msg);
 }
 
 
