@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2023 grunfink et al. / MIT license */
 
-#define VERSION "2.41"
+#define VERSION "2.42"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -27,11 +27,9 @@ int valid_status(int status);
 xs_str *tid(int offset);
 double ftime(void);
 
-void srv_debug(int level, xs_str *str);
-#define srv_log(str) srv_debug(0, str)
-
-int srv_open(char *basedir, int auto_upgrade);
-void srv_free(void);
+void srv_log(xs_str *str);
+#define srv_debug(level, str) do { if (dbglevel >= (level)) \
+    { srv_log((str)); } } while (0)
 
 typedef struct _snac {
     xs_str *uid;        /* uid */
@@ -43,14 +41,18 @@ typedef struct _snac {
     xs_str *md5;        /* actor url md5 */
 } snac;
 
+void snac_log(snac *user, xs_str *str);
+#define snac_debug(user, level, str) do { if (dbglevel >= (level)) \
+    { snac_log((user), (str)); } } while (0)
+
+int srv_open(char *basedir, int auto_upgrade);
+void srv_free(void);
+
 int user_open(snac *snac, const char *uid);
 void user_free(snac *snac);
 xs_list *user_list(void);
 int user_open_by_md5(snac *snac, const char *md5);
 
-void snac_debug(snac *snac, int level, xs_str *str);
-#define snac_log(snac, str) snac_debug(snac, 0, str)
-
 int validate_uid(const char *uid);
 
 xs_str *hash_password(const char *uid, const char *passwd, const char *nonce);
@@ -130,6 +132,7 @@ int is_muted(snac *snac, const char *actor);
 int pin(snac *user, const char *id);
 int unpin(snac *user, const char *id);
 int is_pinned(snac *user, const char *id);
+int is_pinned_by_md5(snac *user, const char *md5);
 xs_list *pinned_list(snac *user);
 
 int limited(snac *user, const char *id, int cmd);
@@ -175,8 +178,8 @@ int instance_unblock(const char *instance);
 
 void enqueue_input(snac *snac, const xs_dict *msg, const xs_dict *req, int retries);
 void enqueue_output_raw(const char *keyid, const char *seckey,
-                        xs_dict *msg, xs_str *inbox, int retries);
-void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries);
+                        xs_dict *msg, xs_str *inbox, int retries, int p_status);
+void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries, int p_status);
 void enqueue_output_by_actor(snac *snac, xs_dict *msg, const xs_str *actor, int retries);
 void enqueue_email(xs_str *msg, int retries);
 void enqueue_telegram(const xs_str *msg, const char *bot, const char *chat_id);
@@ -259,6 +262,7 @@ int activitypub_post_handler(const xs_dict *req, const char *q_path,
 
 xs_str *not_really_markdown(const char *content, xs_list **attach);
 xs_str *sanitize(const char *content);
+xs_str *encode_html_strict(const char *str);
 xs_str *encode_html(const char *str);
 
 xs_str *html_timeline(snac *user, const xs_list *list, int local, int skip, int show, int show_more);
